#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <mmsystem.h>
#include <math.h>
#include "minifmod.h"

class Buffer
{
private:
	CHAR_INFO* Buf;
	int w,h;

public:
	const int Width() const {return w;}
	const int Height() const {return h;}

	CHAR_INFO* operator[](size_t j) const
	{
		return (Buf+j*w);
	}

	void DeAllocate()
	{
		if (Buf)
		{
			delete[] Buf;
			Buf = NULL;
		}
	}

	void Allocate(int width, int height)
	{
		DeAllocate();
		w = width;
		h = height;
		Buf = new CHAR_INFO[w*h];
	}

	void Copy(const Buffer &Source, int originx=0, int originy=0)
	{
		for (int j=0 ; j<Source.Height() ; j++)
			for (int i=0 ; i<Source.Width() ; i++)			
				if (i+originx<w && j+originy<h) 
					Buf[(j+originy)*w+i+originx] = Source[j][i];
	}

	void Clear()
	{
		memset(Buf, 0, h*w*sizeof(CHAR_INFO));
	}

	void Colorize(WORD color)
	{
		for (int i=0 ; i<w*h ; i++) Buf[i].Attributes = color;
	}

	Buffer(int width=0, int height=0):w(width),h(height)
	{
		Buf = NULL;
		Allocate(w,h);
	}
	
	Buffer(const Buffer &T)
	{
		Buf = NULL;
		Allocate(T.Width(), T.Height());
		Copy(T);
	}
	operator=(const Buffer &T)
	{
		Allocate(T.Width(),T.Height());
		Copy(T);
	}
	~Buffer()
	{
		DeAllocate();
	}

	void OuputChar(HANDLE Console)
	{		
		COORD startcoords = {0,0};
		COORD size = {this->Width(),this->Height()};
		SMALL_RECT rect = {0,0,this->Width(),this->Height()};
		WriteConsoleOutput(Console,Buf,size,startcoords,&rect);
	}
	void LoadFromFile(char* file)
	{
		int i,j;
		FILE* File = fopen(file,"rb");
		if (!File) return;
		
		DeAllocate();
		fseek(File,0,SEEK_END);
		long size = ftell(File);
		
		char* Buftemp = new char[size];
		fseek(File,0,SEEK_SET);
		fread(Buftemp, 1, size, File);
		fclose(File);
		
		w = int(strchr(Buftemp,'\n')-Buftemp)-1;
		h = size/w;
		Buf = new CHAR_INFO[w*h];
		for (i=0 ; i<h ; i++) 
			for (j=0 ; j<w ; j++) 
			{
				Buf[i*(w)+j].Char.AsciiChar = (CHAR) Buftemp[i*(w+2)+j];
				Buf[i*(w)+j].Attributes = 15;
			}
		delete[] Buftemp;
	}

	void CreateFromText(char* s, char* fontname, int height, int weight)
	{
		HFONT font = CreateFont(
		  height,						// logical height of font
		  0,							// logical average character width
		  0,					        // angle of escapement
		  0,							// base-line orientation angle
		  weight,						// font weight
		  FALSE,						// italic attribute flag
		  FALSE,						// underline attribute flag
		  FALSE,						// strikeout attribute flag
		  DEFAULT_CHARSET,					// character set identifier
		  OUT_TT_PRECIS,				// output precision
		  CLIP_DEFAULT_PRECIS,			// clipping precision
		  PROOF_QUALITY,				// output quality
		  DEFAULT_PITCH|FF_DONTCARE,	// pitch and family
		  fontname						// pointer to typeface name string
		);

		HDC cdc = CreateCompatibleDC(NULL);

		HFONT oldfont = (HFONT)SelectObject(cdc,font);


		SIZE size;
		GetTextExtentPoint32(cdc, s, strlen(s), &size);

		HBITMAP bmp = CreateCompatibleBitmap(cdc, size.cx*2, size.cy);

		HBITMAP oldbmp = (HBITMAP)SelectObject(cdc, bmp);

		SetTextColor(cdc, 0xFFFFFFFF);
		int res = TextOut(cdc, 0, 0, s, strlen(s));

		Allocate(size.cx, size.cy);

		for (int j=0 ; j<h ; j++)
			for (int i=0 ; i<w ; i++)
			{
				Buf[j*w+i].Char.AsciiChar = (GetPixel(cdc,i,j)?' ':'*');
				Buf[j*w+i].Attributes = 15;
			}
 

		
		SelectObject(cdc, oldbmp);
		SelectObject(cdc, oldfont);

		DeleteDC(cdc);

	}

};


template <class T>
T mod(const T &a, const T &b)
{
	return ((a%b>=0)?(a%b) : (a%b+b) );
}

void Rotozoom(const Buffer &Source, Buffer &Cible, double centrex, double centrey, double angle, double zoom);
void Loupe(const Buffer &Source, Buffer &Cible, double centrex, double centrey, double rayon, double zoom);
void Vortex(const Buffer &Source, Buffer &Cible, double centrex, double centrey, double rayon, double strength);

HANDLE InitConsole(int width, int height, char* title);